@@ -213,6 +213,14 @@ def build_rule(table='filter', chain=None, command=None, position='', full=None,
     To pass in jump options that doesn't take arguments, pass in an empty
     string.
 
+    .. note::
+
+        Whereas iptables will accept ``-p``, ``--proto[c[o[l]]]`` as synonyms
+        of ``--protocol``, if ``--proto`` appears in an iptables command after
+        the appearance of ``-m policy``, it is interpreted as the ``--proto``
+        option of the policy extension (see the iptables-extensions(8) man
+        page).
+
     CLI Examples:
 
     .. code-block:: bash
@@ -243,7 +251,6 @@ def build_rule(table='filter', chain=None, command=None, position='', full=None,
         salt '*' iptables.build_rule filter INPUT command=I position=3 \\
             full=True match=state connstate=RELATED,ESTABLISHED jump=ACCEPT \\
             family=ipv6
-
     '''
     if 'target' in kwargs:
         kwargs['jump'] = kwargs.pop('target')
@@ -257,7 +264,7 @@ def build_rule(table='filter', chain=None, command=None, position='', full=None,
             del kwargs[ignore]
 
     rule = []
-    proto = False
+    protocol = False
     bang_not_pat = re.compile(r'(!|not)\s?')
 
     def maybe_add_negation(arg):
@@ -281,12 +288,15 @@ def build_rule(table='filter', chain=None, command=None, position='', full=None,
         rule.append('{0}-o {1}'.format(maybe_add_negation('of'), kwargs['of']))
         del kwargs['of']
 
-    for proto_arg in ('protocol', 'proto'):
-        if proto_arg in kwargs:
-            if not proto:
-                rule.append('{0}-p {1}'.format(maybe_add_negation(proto_arg), kwargs[proto_arg]))
-                proto = True
-            del kwargs[proto_arg]
+    if 'proto' in kwargs and kwargs.get('match') != 'policy':
+        kwargs['protocol'] = kwargs['proto']
+        del kwargs['proto']
+        # Handle the case 'proto' in kwargs and kwargs.get('match') == 'policy' below
+    if 'protocol' in kwargs:
+        if not protocol:
+            rule.append('{0}-p {1}'.format(maybe_add_negation('protocol'), kwargs['protocol']))
+            protocol = True
+        del kwargs['protocol']
 
     if 'match' in kwargs:
         match_value = kwargs['match']
@@ -297,6 +307,9 @@ def build_rule(table='filter', chain=None, command=None, position='', full=None,
             if 'name_' in kwargs and match.strip() in ('pknock', 'quota2', 'recent'):
                 rule.append('--name {0}'.format(kwargs['name_']))
                 del kwargs['name_']
+        if 'proto' in kwargs and kwargs.get('match') == 'policy':
+            rule.append('{0}--proto {1}'.format(maybe_add_negation('proto'), kwargs['proto']))
+            del kwargs['proto']
         del kwargs['match']
 
     if 'match-set' in kwargs:
@@ -330,8 +343,8 @@ def build_rule(table='filter', chain=None, command=None, position='', full=None,
         if multiport_arg in kwargs:
             if '-m multiport' not in rule:
                 rule.append('-m multiport')
-                if not proto:
-                    return 'Error: proto must be specified'
+                if not protocol:
+                    return 'Error: protocol must be specified'
 
             mp_value = kwargs[multiport_arg]
             if isinstance(mp_value, list):
@@ -1042,9 +1055,9 @@ def _parse_conf(conf_file=None, in_mem=False, family='ipv4'):
 
 def _parser():
     '''
-    This function contains _all_ the options I could find in man 8 iptables,
-    listed in the first section that I found them in. They will not all be used
-    by all parts of the module; use them intelligently and appropriately.
+    This function attempts to list all the options documented in the
+    iptables(8) and iptables-extensions(8) man pages.  They will not all be
+    used by all parts of the module; use them intelligently and appropriately.
     '''
     add_arg = None
     if sys.version.startswith('2.6'):
