@@ -8,21 +8,25 @@ Homebrew for macOS
     *'pkg.install' is not available*), see :ref:`here
     <module-provider-override>`.
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 
 # Import python libs
 import copy
-import json
+import functools
 import logging
 
 # Import salt libs
-import salt.utils
+import salt.utils.data
+import salt.utils.functools
+import salt.utils.json
+import salt.utils.path
+import salt.utils.pkg
+import salt.utils.versions
 from salt.exceptions import CommandExecutionError, MinionError
-import salt.ext.six as six
-from salt.ext.six.moves import zip
 
 # Import third party libs
-import json
+from salt.ext import six
+from salt.ext.six.moves import zip
 
 log = logging.getLogger(__name__)
 
@@ -35,7 +39,7 @@ def __virtual__():
     Confine this module to Mac OS with Homebrew.
     '''
 
-    if salt.utils.which('brew') and __grains__['os'] == 'MacOS':
+    if salt.utils.path.which('brew') and __grains__['os'] == 'MacOS':
         return __virtualname__
     return (False, 'The brew module could not be loaded: brew not found or grain os != MacOS')
 
@@ -60,7 +64,7 @@ def _tap(tap, runas=None):
     try:
         _call_brew(cmd)
     except CommandExecutionError:
-        log.error('Failed to tap "{0}"'.format(tap))
+        log.error('Failed to tap "%s"', tap)
         return False
 
     return True
@@ -103,9 +107,9 @@ def list_pkgs(versions_as_list=False, **kwargs):
 
         salt '*' pkg.list_pkgs
     '''
-    versions_as_list = salt.utils.is_true(versions_as_list)
+    versions_as_list = salt.utils.data.is_true(versions_as_list)
     # not yet implemented or not applicable
-    if any([salt.utils.is_true(kwargs.get(x))
+    if any([salt.utils.data.is_true(kwargs.get(x))
             for x in ('removed', 'purge_desired')]):
         return {}
 
@@ -125,7 +129,8 @@ def list_pkgs(versions_as_list=False, **kwargs):
             name_and_versions = line.split(' ')
             name = name_and_versions[0]
             installed_versions = name_and_versions[1:]
-            newest_version = sorted(installed_versions, cmp=salt.utils.version_cmp).pop()
+            key_func = functools.cmp_to_key(salt.utils.versions.version_cmp)
+            newest_version = sorted(installed_versions, key=key_func).pop()
         except ValueError:
             continue
         __salt__['pkg_resource.add_pkg'](ret, name, newest_version)
@@ -168,7 +173,7 @@ def latest_version(*names, **kwargs):
         salt '*' pkg.latest_version <package name>
         salt '*' pkg.latest_version <package1> <package2> <package3>
     '''
-    refresh = salt.utils.is_true(kwargs.pop('refresh', True))
+    refresh = salt.utils.data.is_true(kwargs.pop('refresh', True))
     if refresh:
         refresh_db()
 
@@ -184,7 +189,7 @@ def latest_version(*names, **kwargs):
         return versions_dict
 
 # available_version is being deprecated
-available_version = salt.utils.alias_function(latest_version, 'available_version')
+available_version = salt.utils.functools.alias_function(latest_version, 'available_version')
 
 
 def remove(name=None, pkgs=None, **kwargs):
@@ -235,7 +240,7 @@ def remove(name=None, pkgs=None, **kwargs):
 
     __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs()
-    ret = salt.utils.compare_dicts(old, new)
+    ret = salt.utils.data.compare_dicts(old, new)
 
     if errors:
         raise CommandExecutionError(
@@ -256,6 +261,8 @@ def refresh_db():
 
         salt '*' pkg.refresh_db
     '''
+    # Remove rtag file to keep multiple refreshes from happening in pkg states
+    salt.utils.pkg.clear_rtag(__opts__)
     cmd = 'brew update'
     if _call_brew(cmd)['retcode']:
         log.error('Failed to update')
@@ -282,9 +289,10 @@ def _info(*pkgs):
     cmd = 'brew info --json=v1 {0}'.format(' '.join(pkgs))
     brew_result = _call_brew(cmd)
     if brew_result['retcode']:
-        log.error('Failed to get info about packages: {0}'.format(' '.join(pkgs)))
+        log.error('Failed to get info about packages: %s',
+                  ' '.join(pkgs))
         return {}
-    output = json.loads(brew_result['stdout'])
+    output = salt.utils.json.loads(brew_result['stdout'])
     return dict(zip(pkgs, output))
 
 
@@ -386,7 +394,7 @@ def install(name=None, pkgs=None, taps=None, options=None, **kwargs):
 
     __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs()
-    ret = salt.utils.compare_dicts(old, new)
+    ret = salt.utils.data.compare_dicts(old, new)
 
     if errors:
         raise CommandExecutionError(
@@ -414,7 +422,7 @@ def list_upgrades(refresh=True, **kwargs):  # pylint: disable=W0613
     ret = {}
 
     try:
-        data = json.loads(res['stdout'])
+        data = salt.utils.json.loads(res['stdout'])
     except ValueError as err:
         msg = 'unable to interpret output from "brew outdated": {0}'.format(err)
         log.error(msg)
@@ -467,13 +475,13 @@ def upgrade(refresh=True):
 
     old = list_pkgs()
 
-    if salt.utils.is_true(refresh):
+    if salt.utils.data.is_true(refresh):
         refresh_db()
 
     result = _call_brew('brew upgrade', failhard=False)
     __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs()
-    ret = salt.utils.compare_dicts(old, new)
+    ret = salt.utils.data.compare_dicts(old, new)
 
     if result['retcode'] != 0:
         raise CommandExecutionError(
