@@ -1,20 +1,32 @@
 # -*- coding: utf-8 -*-
-from __future__ import absolute_import
-from __future__ import print_function
-
+'''
+Functions for creating and working with job IDs
+'''
+from __future__ import absolute_import, print_function, unicode_literals
 from calendar import month_abbr as months
 import datetime
 import hashlib
 import os
 
+import salt.utils.stringutils
 from salt.ext import six
 
+LAST_JID_DATETIME = None
 
-def gen_jid():
+
+def gen_jid(opts):
     '''
     Generate a jid
     '''
-    return '{0:%Y%m%d%H%M%S%f}'.format(datetime.datetime.now())
+    global LAST_JID_DATETIME  # pylint: disable=global-statement
+
+    if not opts.get('unique_jid', False):
+        return '{0:%Y%m%d%H%M%S%f}'.format(datetime.datetime.now())
+    jid_dt = datetime.datetime.now()
+    if LAST_JID_DATETIME and LAST_JID_DATETIME >= jid_dt:
+        jid_dt = LAST_JID_DATETIME + datetime.timedelta(microseconds=1)
+    LAST_JID_DATETIME = jid_dt
+    return '{0:%Y%m%d%H%M%S%f}_{1}'.format(jid_dt, os.getpid())
 
 
 def is_jid(jid):
@@ -23,10 +35,10 @@ def is_jid(jid):
     '''
     if not isinstance(jid, six.string_types):
         return False
-    if len(jid) != 20:
+    if len(jid) != 20 and (len(jid) <= 21 or jid[20] != '_'):
         return False
     try:
-        int(jid)
+        int(jid[:20])
         return True
     except ValueError:
         return False
@@ -36,8 +48,8 @@ def jid_to_time(jid):
     '''
     Convert a salt job id into the time when the job was invoked
     '''
-    jid = str(jid)
-    if len(jid) != 20:
+    jid = six.text_type(jid)
+    if len(jid) != 20 and (len(jid) <= 21 or jid[20] != '_'):
         return ''
     year = jid[:4]
     month = jid[4:6]
@@ -45,7 +57,7 @@ def jid_to_time(jid):
     hour = jid[8:10]
     minute = jid[10:12]
     second = jid[12:14]
-    micro = jid[14:]
+    micro = jid[14:20]
 
     ret = '{0}, {1} {2} {3}:{4}:{5}.{6}'.format(year,
                                                 months[int(month)],
@@ -65,7 +77,7 @@ def format_job_instance(job):
            'Arguments': list(job.get('arg', [])),
            # unlikely but safeguard from invalid returns
            'Target': job.get('tgt', 'unknown-target'),
-           'Target-type': job.get('tgt_type', []),
+           'Target-type': job.get('tgt_type', 'list'),
            'User': job.get('user', 'root')}
 
     if 'metadata' in job:
@@ -102,10 +114,9 @@ def jid_dir(jid, job_dir=None, hash_type='sha256'):
     Return the jid_dir for the given job id
     '''
     if not isinstance(jid, six.string_types):
-        jid = str(jid)
-    if six.PY3:
-        jid = jid.encode('utf-8')
-    jhash = getattr(hashlib, hash_type)(jid).hexdigest()
+        jid = six.text_type(jid)
+    jhash = getattr(hashlib, hash_type)(
+        salt.utils.stringutils.to_bytes(jid)).hexdigest()
 
     parts = []
     if job_dir is not None:
